/**
 * @name Strip html tags in field and merges descr.
 * @author 2conciliate Business Solutions AB, Sebastian Skoglund
 * @description 2017-10-02
 * @runtime 1
 *
 */

////////////////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////////////////

var htmlStripDataType = "se.conciliate.simple.text";
var htmlStripPattern = /<html[^>]*>((.|[\n\r])*)<\/html>/im;
var descriptionName = "Description";
var MergeDataType = "se.conciliate.rich.text";
var defaultDescriptionUuid = "f21aef69-9772-49bd-b6c1-4f3e55fc3887";
var MergePattern = /<body[^>]*>((.|[\n\r])*)<\/body>/im;


function running() {
    if (Runtime.isRunning() == false) {
        throw "Script aborted";
    }
}

var each = function (f, coll) {
    var i, ii;

    if (typeof (f) !== "function") {
        throw "f must be a function";
    }

    if (typeof (coll) === "undefined") {
        throw "coll can not be undefined";
    }

    if (typeof (coll.hasNext) === "function" && typeof (coll.next) === "function") {
        while (coll.hasNext()) {
            f(coll.next());
        }
    } else if (typeof (coll.iterator) === "function") {
        return each(f, coll.iterator());
    } else if (typeof (coll.length) === "number") {
        i = 0, ii = coll.length;
        for (; i < ii; i++) {
            f(coll[i]);
        }
    }
};

(function (workspace, repository) {
    var descriptions = new java.util.ArrayList();
    var fields = new java.util.ArrayList();
    var visited = new java.util.HashSet();
    var defaultDescriptionAttrType = null;
    var attributeTypeIt = repository.findAttributeTypes().iterator();
    while (attributeTypeIt.hasNext()) {
        var at = attributeTypeIt.next();
        if (at.getDataType() === htmlStripDataType) {
            fields.add(at);
        }
        if (at.getUUID() === defaultDescriptionUuid) {
            defaultDescriptionAttrType = at;
        } else if (at.getTitle() === descriptionName && at.getDataType() === MergeDataType) {
            descriptions.add(at);
        }
    }
    if (fields.size() === 0) {
        println("INFO: No text fields found");
    }
    if (defaultDescriptionAttrType === null) {
        throw "could not find any default description";
    }
    if (descriptions.size() === 0) {
        println("INFO: No descriptions to merge from found");
    }

    var inspectSymbol = function (symbol) {
        if (!visited.contains(symbol.getID())) {
            visited.add(symbol.getID());
            symbol = symbol.load();
            var merged = false;
            var changed = false;
            var it;
            if (fields.size() !== 0) {
                it = fields.iterator();
                while (it.hasNext()) {
                    var a = it.next();
                    if (symbol.findAttributeByType(a.getUUID()) !== null) {
                        var d = symbol.getAttribute(a);
                        if (!d.isEmpty()) {
                            if (htmlStripPattern.test(d.getData())) {
                                changed = true;
                                var array_matches = htmlStripPattern.exec(d.getData());
                                d.setData(array_matches[1]);
                            }
                        }
                    }
                }
            }

                
            var attr = null;
            var data = "";
            if (descriptions.size() !== 0) {
                var attrTypeIt = repository.findAttributeTypes(symbol.getType()).iterator();
                while (attrTypeIt.hasNext()) {
                    var at = attrTypeIt.next();
                    if (at.getUUID() === defaultDescriptionUuid) {
                        attr = symbol.getAttribute(defaultDescriptionAttrType);
                    }
                }

                it = descriptions.iterator();
                if (attr !== null && !attr.isEmpty()) {
                    var array_matches = MergePattern.exec(attr.getData());
                    if (array_matches !== null) {
                        data = array_matches[1];
                    }
                }
                while (it.hasNext()) {
                    var a = it.next();
                    if (symbol.findAttributeByType(a.getUUID()) !== null) {
                        if (attr === null) {
                            defaultDescriptionAttrType.connectToSymbolType(symbol.getType());
                            defaultDescriptionAttrType.save();
                            attr = symbol.getAttribute(defaultDescriptionAttrType);
                            println("No default description for symbol type '" + symbol.getType().getID() + "', now connected!");
                        }
                        var d = symbol.getAttribute(a);
                        if (!d.isEmpty()) {
                            var array_matches = MergePattern.exec(d.getData());
                            if (array_matches !== null) {
                                if (array_matches[1].lastIndexOf("#MERGED#", 0) === 0) { //startswith
                                    println("already merged '" + symbol.getTitle() + "'");
                                } else {
                                    merged = true;
                                    if (data.length === 0) {
                                        data = array_matches[1];
                                    } else {
                                        data = data + '<p>' + array_matches[1];
                                    }
                                    d.setData('<html><head></head><body>#MERGED#<p>' + array_matches[1] + '</body></html>')

                                }
                            }
                        }
                    }
                }
            }
            data = '<html><head></head><body>' + data + '</body></html>';

            if (merged || changed) {
                if (!symbol.isRevisionLocked()) {
                    if (changed) {
                        println("stripped html in fields for symbol '" + symbol.getTitle() + "'");
                    }
                    if (merged) {
                        attr.setData(data);
                        println("merged descriptions for symbol '" + symbol.getTitle() + "'");
                    }
                    symbol.save();
                } else {
                    println("WARNING: could not update symbol '" + symbol.getTitle() + "', revision is locked!");
                }
            }
        }
    };

    each(inspectSymbol, workspace.findSymbols());
    
    println("");
    println("All Done!");
    println("");


})(MTStore.getCurrentWorkspace(), MTStore.getCurrentRepository());
